using System;
using Microsoft.SPOT;
using System.IO;

namespace CodingSmackdown.Services
{
    public class Settings
    {
        public const string SETTINGS_FILE = @"\SD\settings.txt";
        public const int MINUTES_MULTIPLIER = 60000;

        private int timeZoneOffset;

        public int TimeZoneOffset
        {
            get { return timeZoneOffset; }
            set { timeZoneOffset = value; }
        }

        private int minutesBetweenReadings;

        public int MinutesBetweenReadings
        {
            get { return minutesBetweenReadings * MINUTES_MULTIPLIER; }
            set { minutesBetweenReadings = value; }
        }

        private int temperatureOffset;

        public int TemperatureOffset
        {
            get { return temperatureOffset; }
            set { temperatureOffset = value; }
        }

        private string historyFilename;

        public string HistoryFilename
        {
            get { return historyFilename; }
            set { historyFilename = value; }
        }

        private string ntpServerName;

        public string NTPServerName
        {
            get { return ntpServerName; }
            set { ntpServerName = value; }
        }
        
        private int minutesBetweenNTPUpdate;

        public int MinutesBetweenNTPUpdate
        {
            get { return minutesBetweenNTPUpdate * MINUTES_MULTIPLIER; }
            set { minutesBetweenNTPUpdate = value; }
        }
        
        private string netBiosName;

        public string NetBiosName
        {
            get { return netBiosName; }
            set { netBiosName = value; }
        }

        private bool enableDHCP;

        public bool EnableDHCP
        {
            get { return enableDHCP; }
            set { enableDHCP = value; }
        }

        private string staticIPAddress;

        public string StaticIPAddress
        {
            get { return staticIPAddress; }
            set { staticIPAddress = value; }
        }

        private string subnetMask;

        public string SubnetMask
        {
            get { return subnetMask; }
            set { subnetMask = value; }
        }

        private string defaultGateway;

        public string DefaultGateway
        {
            get { return defaultGateway; }
            set { defaultGateway = value; }
        }

        private string primaryDNSAddress;

        public string PrimaryDNSAddress
        {
            get { return primaryDNSAddress; }
            set { primaryDNSAddress = value; }
        }

        private string secondaryDNSAddress;

        public string SecondaryDNSAddress
        {
            get { return secondaryDNSAddress; }
            set { secondaryDNSAddress = value; }
        }

        public void loadSettings()
        {
            try
            {
                using (StreamReader file = new StreamReader(SETTINGS_FILE))
                {
                    string line;
                    string[] setting;
                    // Read and display lines from the file until the end of 
                    // the file is reached.
                    while ((line = file.ReadLine()) != null)
                    {
                        if (line != String.Empty)
                        {
                            setting = line.Split('=');
                            if((setting != null) && (setting.Length > 1))
                            {
                                switch(setting[0])
                                {
                                    case "TimeZoneOffset":
                                        TimeZoneOffset = Convert.ToInt32(setting[1]);
                                        break;
                                    case "MinutesBetweenReadings":
                                        MinutesBetweenReadings = Convert.ToInt32(setting[1]);
                                        break;
                                    case "TemperatureOffset":
                                        TemperatureOffset = Convert.ToInt32(setting[1]);
                                        break;
                                    case "HistoryFilename":
                                        HistoryFilename = setting[1];
                                        break;
                                    case "NTPServerName":
                                        NTPServerName = setting[1];
                                        break;
                                    case "MinutesBetweenNTPUpdate":
                                        MinutesBetweenNTPUpdate = Convert.ToInt32(setting[1]);
                                        break;
                                    case "NetBiosName":
                                        NetBiosName = setting[1];
                                        break;
                                    case "EnableDHCP":
                                        if(setting[1].ToLower().Equals("true"))
                                        {
                                            EnableDHCP = true;
                                        }
                                        else
                                        {
                                            EnableDHCP = false;
                                        }
                                        break;
                                    case "StaticIPAddress":
                                        StaticIPAddress = setting[1];
                                        break;
                                    case "SubnetMask":
                                        SubnetMask = setting[1];
                                        break;
                                    case "DefaultGateway":
                                        DefaultGateway = setting[1];
                                        break;
                                    case "PrimaryDNSAddress":
                                        PrimaryDNSAddress = setting[1];
                                        break;
                                    case "SecondaryDNSAddress":
                                        SecondaryDNSAddress = setting[1];
                                        break;
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        public void saveSettings()
        {
            try
            {
                using (StreamWriter file = new StreamWriter(SETTINGS_FILE, false))
                {
                    file.Write("TimeZoneOffset=");
                    file.WriteLine(timeZoneOffset.ToString());

                    file.Write("MinutesBetweenReadings=");
                    file.WriteLine(minutesBetweenReadings.ToString());

                    file.Write("TemperatureOffset=");
                    file.WriteLine(temperatureOffset.ToString());

                    file.Write("HistoryFilename=");
                    file.WriteLine(historyFilename);

                    file.Write("NTPServerName=");
                    file.WriteLine(ntpServerName);

                    file.Write("MinutesBetweenNTPUpdate=");
                    file.WriteLine(minutesBetweenNTPUpdate.ToString());

                    file.Write("NetBiosName=");
                    file.WriteLine(netBiosName);

                    file.Write("EnableDHCP=");
                    file.WriteLine(EnableDHCP.ToString());

                    file.Write("StaticIPAddress=");
                    file.WriteLine(StaticIPAddress);

                    file.Write("SubnetMask=");
                    file.WriteLine(SubnetMask);

                    file.Write("DefaultGateway=");
                    file.WriteLine(DefaultGateway);

                    file.Write("PrimaryDNSAddress=");
                    file.WriteLine(PrimaryDNSAddress);

                    file.Write("SecondaryDNSAddress=");
                    file.WriteLine(SecondaryDNSAddress);
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }
    }
}
